#include <iostream>
#include <vector>
#include "histogram.h"
#include "svg.h"
#include "text.h"
#include <curl/curl.h>
#include <sstream>
#include <string>

using namespace std;

struct Input
{
    vector<double> numbers;
    size_t bin_count{};
};

Input input_data(istream &in, bool promt)
{

    size_t number_count, bin_count;

    if (promt)
    {
        cerr << "Enter number count: ";
    }
    in >> number_count;

    Input input;

    input.numbers.resize(number_count);
    if (promt)
    {
        cerr << "Enter numbers: ";
    }

    for (size_t i = 0; i < number_count; i++)
    {
        in >> input.numbers[i];
    }

    if (promt)
    {
        cerr << "Count of baskets: ";
    }
    in >> input.bin_count;

    return input;
}
size_t
write_data(void *items, size_t item_size, size_t item_count, void *ctx)
{
    size_t data_size = item_count * item_size;

    stringstream *buffer = reinterpret_cast<stringstream *>(ctx);
    char *item = reinterpret_cast<char *>(items);
    //(*buffer).write(item, data_size);
    buffer->write(item, data_size);

    return data_size;
}

Input download(const string &address)
{
    stringstream buffer;

    // address.c_str();

    curl_global_init(CURL_GLOBAL_ALL);
    {
        CURL *curl = curl_easy_init();
        if (curl)
        {
            CURLcode res;
            double total;
            curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
            curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
            curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);

            res = curl_easy_perform(curl);

            if (res != CURLE_OK)
            {

                fprintf(stderr, "curl_easy_perform() failed: %s\n",
                        curl_easy_strerror(res));
                exit(1);
            }

            if (CURLE_OK == res)
            {
                res = curl_easy_getinfo(curl, CURLINFO_TOTAL_TIME, &total);
                cerr << "Time spending for downloading file: " << total << endl;
            }
            curl_easy_cleanup(curl);
        }
    }

    return input_data(buffer, false);
}

int main(int argc, char *argv[])
{
    Input input;
    if (argc > 1)
    {
        input = download(argv[1]);
    }
    else
    {
        input = input_data(cin, true);
    }

    const auto bins = make_histogram(input.numbers, input.bin_count);
    show_histogram_svg(bins);
}